@@ -5,10 +5,51 @@ use Exporter ();
 use IO::File ();
 use File::Copy ();
 
-*import = \&Exporter::import;
+$Apache::ExtUtils::VERSION = '1.04';
+
+my @config_export = qw(%Config ldopts ccopts);
 @Apache::ExtUtils::EXPORT = qw(command_table);
+@Apache::ExtUtils::EXPORT_OK = (qw(pm), @config_export);
+
 my $errsv = "";
 
+sub import {
+    my $class = shift;
+    my $config_export = join '|', @config_export;
+    for (@_) {
+	#perl -Mlib=lib -MApache::ExtUtils=%Config -e 'print $Config{cc}'
+	if (/$config_export/o) {
+	    require Config;
+	    *Apache::ExtUtils::Config = \%Config::Config;
+	    Config_pm_fixup();
+	    require ExtUtils::Embed;
+	}
+    }
+    local $Exporter::ExportLevel = 1;
+    $class->Exporter::import(@_);
+}
+
+*ldopts = \&ExtUtils::Embed::ldopts;
+*ccopts = \&ExtUtils::Embed::ccopts;
+
+sub Config_pm_fixup {
+    eval { require Apache::MyConfig; };
+    my %config_fixups = (
+       ccdlflags => sub { s/-R\s+/-R/; },
+       ccflags => sub {
+           unless ($Apache::MyConfig::Setup{PERL_USELARGEFILES}) {
+               s/-D_LARGEFILE_SOURCE\s+-D_FILE_OFFSET_BITS=\d+//;
+           }
+       },
+    );
+
+    while (my($key, $sub) = each %config_fixups) {
+	local $_ = $Config::Config{$key};
+	$sub->();
+	(tied %Config::Config)->{$key} = $_;
+    }
+}
+
 sub command_table {
     my($class, $cmds);
     if(@_ == 2) {
@@ -81,6 +122,9 @@ EOF
 sub xs_cmd_table {
     my($self, $class, $cmds) = @_;
     (my $modname = $class) =~ s/::/__/g;
+    (my $pmname = $class) =~ s,::,/,g;
+    $pmname .= '.pm';
+
     my $cmdtab = "";
     my $infos = "";
 
@@ -159,7 +203,7 @@ static mod_perl_perl_dir_config *newPerlConfig(pool *p)
 	(mod_perl_perl_dir_config *)
 	    palloc(p, sizeof (mod_perl_perl_dir_config));
     cld->obj = Nullsv;
-    cld->class = "$class";
+    cld->pclass = "$class";
     register_cleanup(p, cld, perl_perl_cmd_cleanup, null_cleanup);
     return cld;
 }
@@ -211,16 +255,125 @@ module MODULE_VAR_EXPORT XS_${modname} = {
     NULL,   /* [1] post read_request handling */
 };
 
+#define this_module "$pmname"
+
+static void remove_module_cleanup(void *data)
+{
+    if (find_linked_module("$class")) {
+        /* need to remove the module so module index is reset */
+        remove_module(&XS_${modname});
+    }
+    if (data) {
+        /* make sure BOOT section is re-run on restarts */
+        (void)hv_delete(GvHV(incgv), this_module,
+                        strlen(this_module), G_DISCARD);
+         if (dowarn) {
+             /* avoid subroutine redefined warnings */
+             perl_clear_symtab(gv_stashpv("$class", FALSE));
+         }
+    }
+}
+
 MODULE = $class		PACKAGE = $class
 
+PROTOTYPES: DISABLE
+
 BOOT:
     XS_${modname}.name = "$class";
     add_module(&XS_${modname});
     stash_mod_pointer("$class", &XS_${modname});
+    register_cleanup(perl_get_startup_pool(), (void *)1,
+                     remove_module_cleanup, null_cleanup);
 
+void
+END()
+
+    CODE:
+    remove_module_cleanup(NULL);
 EOF
 }
 
+#perl -MApache::ExtUtils=pm -e pm -- Apache::Foo
+sub pm {
+    my($class) = @_ ? @_ : @ARGV;
+    (my $name = $class) =~ s/.*::(\w+)$/$1/;
+    write_pm($class, $name);
+    write_makepl($class, $name);
+}
+
+sub outfh {
+    my($file) = @_;
+
+    my $fh = local *FH;
+    if (-e $file) {
+	die "$file exists";
+    }
+    open $fh, ">$file" or die "open $file: $!";
+    print STDERR "writing $file\n";
+    return $fh;
+}
+
+sub write_pm {
+    my($class, $name) = @_;
+    my $fh = outfh("$name.pm");
+    print $fh <<EOF;
+package $class;
+
+use strict;
+use Apache::ModuleConfig ();
+use DynaLoader ();
+ 
+if(\$ENV{MOD_PERL}) {
+    no strict;
+    \$VERSION = '1.00';
+    \@ISA = qw(DynaLoader);
+     __PACKAGE__->bootstrap(\$VERSION);
+}
+
+sub DirectiveName (\$\$\$) {
+     my(\$cfg, \$parms, \$arg) = \@_;
+     my \$scfg = Apache::ModuleConfig->get(\$parms->server);
+
+}
+
+1;
+__END__
+EOF
+  close $fh or die $!;
+}
+
+sub write_makepl {
+    my($class, $name) = @_;
+
+    my $fh = outfh("Makefile.PL");
+    print $fh <<EOF;
+package $class;
+
+use ExtUtils::MakeMaker;
+
+use Apache::ExtUtils qw(command_table);
+use Apache::src ();
+
+my \@directives = ( 
+	 	   { 
+		    name     => 'DirectiveName',
+		    errmsg   => 'the syntax error message',
+		    args_how => 'TAKE1',
+		    req_override => 'OR_ALL',
+		   }
+		  );
+
+command_table(\\\@directives);
+
+WriteMakefile(
+     'NAME'	=> __PACKAGE__,
+     'VERSION_FROM' => '$name.pm',
+     'INC'	=> Apache::src->new->inc,
+ );
+EOF
+  close $fh or die $!;
+}
+
 1;
 
 __END__
