@@ -19,32 +19,24 @@ unless (defined $Apache::Registry::MarkLine) {
 $Debug ||= 0;
 my $Is_Win32 = $^O eq "MSWin32";
 
-@Apache::PerlRun::ISA = qw(Apache);
-
 sub new {
     my($class, $r) = @_;
-    return $r unless ref($r) eq "Apache";
-    if(ref $r) {
-	$r->request($r);
-    }
-    else {
-	$r = Apache->request;
-    }
     my $filename = $r->filename;
     $r->warn("Apache::PerlRun->new for $filename in process $$")
 	if $Debug && $Debug & 4;
 
-    bless {
-	'_r' => $r,
-    }, $class;
+    return bless {r=>$r}, $class;
 }
 
 sub can_compile {
     my($pr) = @_;
-    my $filename = $pr->filename;
+    my $r = $pr->{r};
+    my $filename = $r->filename;
+#XXX reported problems with $r->finfo
+#    if (-r $r->finfo && -s _) {
     if (-r $filename && -s _) {
-	if (!($pr->allow_options & OPT_EXECCGI)) {
-	    $pr->log_reason("Options ExecCGI is off in this directory",
+	if (!($r->allow_options & OPT_EXECCGI)) {
+	    $r->log_reason("Options ExecCGI is off in this directory",
 			   $filename);
 	    return FORBIDDEN;
  	}
@@ -52,7 +44,7 @@ sub can_compile {
 	    return DECLINED;
 	}
 	unless (-x _ or $Is_Win32) {
-	    $pr->log_reason("file permissions deny server execution",
+	    $r->log_reason("file permissions deny server execution",
 			   $filename);
 	    return FORBIDDEN;
 	}
@@ -60,12 +52,12 @@ sub can_compile {
 	$pr->{'mtime'} = -M _;
 	return wantarray ? (OK, $pr->{'mtime'}) : OK;
     }
+    $r->log_error("$filename not found or unable to stat");
     return NOT_FOUND;
 }
 
 sub mark_line {
-    my($pr) = @_;
-    my $filename = $pr->filename;
+    my $filename = shift->{r}->filename;
     return $Apache::Registry::MarkLine ?
 	"\n#line 1 $filename\n" : "";
 }
@@ -104,7 +96,7 @@ sub should_compile {
       $Apache::Registry->{$package}{'mtime'} <= $mtime);
 }
 
-sub update_mtime {
+sub set_mtime {
     my($pr, $mtime, $package) = @_;
     $mtime   ||= $pr->{'mtime'};
     $package ||= $pr->{'namespace'};
@@ -114,26 +106,28 @@ sub update_mtime {
 sub compile {
     my($pr, $eval) = @_;
     $eval ||= $pr->{'sub'};
-    $pr->clear_rgy_endav;
-    $pr->log_error("Apache::PerlRun->compile") if $Debug && $Debug & 4;
+    my $r = $pr->{r};
+    $r->clear_rgy_endav;
+    $r->log_error("Apache::PerlRun->compile") if $Debug && $Debug & 4;
     Apache->untaint($$eval);
     {
 	no strict; #so eval'd code doesn't inherit our bits
 	eval $$eval;
     }
-    $pr->stash_rgy_endav;
+    $r->stash_rgy_endav;
     return $pr->error_check;
 }
 
 sub run {
     my $pr = shift;
     my $package = $pr->{'namespace'};
+    my $r = $pr->{r};
 
     my $rc = OK;
     my $cv = \&{"$package\::handler"};
 
     my $oldwarn = $^W;
-    eval { $rc = &{$cv}($pr, @_) } if $pr->seqno;
+    eval { $rc = &{$cv}($r, @_) } if $r->seqno;
     $pr->{status} = $rc;
     $^W = $oldwarn;
 
@@ -141,11 +135,11 @@ sub run {
     if($@) {
 	$errsv = $@;
 	$@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks
-	$@{$pr->uri} = $errsv;
+	$@{$r->uri} = $errsv;
     }
 
     if($errsv) {
-	$pr->log_error($errsv);
+	$r->log_error($errsv);
 	return SERVER_ERROR;
     }
 
@@ -153,24 +147,25 @@ sub run {
 }
 
 sub status {
-    shift->{'_r'}->status;
+    shift->{r}->status;
 }
 
 sub namespace_from {
     my($pr) = @_;
+    my $r = $pr->{r};
 
-    my $uri = $pr->uri; 
+    my $uri = $r->uri;
 
-    $pr->log_error(sprintf "Apache::PerlRun->namespace escaping %s",
+    $r->log_error(sprintf "Apache::PerlRun->namespace escaping %s",
 		  $uri) if $Debug && $Debug & 4;
 
-    my $path_info = $pr->path_info;
+    my $path_info = $r->path_info;
     my $script_name = $path_info && $uri =~ /$path_info$/ ?
 	substr($uri, 0, length($uri)-length($path_info)) :
 	$uri;
 
-    if ($Apache::Registry::NameWithVirtualHost && $pr->server->is_virtual) {
-	my $name = $pr->get_server_name;
+    if ($Apache::Registry::NameWithVirtualHost && $r->server->is_virtual) {
+	my $name = $r->get_server_name;
 	$script_name = join "", $name, $script_name if $name;
     }
 
@@ -200,7 +195,7 @@ sub namespace {
  
     $root ||= "Apache::ROOT";
 
-    $pr->log_error("Apache::PerlRun->namespace: package $root$script_name")
+    $pr->{r}->log_error("Apache::PerlRun->namespace: package $root$script_name")
 	if $Debug && $Debug & 4;
 
     $pr->{'namespace'} = $root.$script_name;
@@ -209,14 +204,14 @@ sub namespace {
 
 sub readscript {
     my $pr = shift;
-    $pr->{'code'} = $pr->slurp_filename;
+    $pr->{'code'} = $pr->{r}->slurp_filename;
 }
 
 sub error_check {
     my $pr = shift;
     if ($@ and substr($@,0,4) ne " at ") {
-	$pr->log_error("PerlRun: `$@'");
-	$@{$pr->uri} = $@;
+	$pr->{r}->log_error("PerlRun: `$@'");
+	$@{$pr->{r}->uri} = $@;
 	$@ = ''; #XXX fix me, if we don't do this Apache::exit() breaks	
 	return SERVER_ERROR;
     }
@@ -257,12 +252,12 @@ sub parse_cmdline {
 
 sub chdir_file {
     my($pr, $dir) = @_;
-    $pr->{'_r'}->chdir_file($dir ? $dir : $pr->filename);
+    my $r = $pr->{r};
+    $r->chdir_file($dir ? $dir : $r->filename);
 }
 
 sub set_script_name {
-    my($pr) = @_;
-    *0 = \$pr->filename;
+    *0 = \(shift->{r}->filename);
 }
 
 sub handler ($$) {
@@ -312,36 +307,55 @@ sub handler ($$) {
     return $rc;
 }
 
+BEGIN {
+    if ($] < 5.006) {
+        $INC{'warnings.pm'} = __FILE__;
+        *warnings::unimport = sub {};
+    }
+}
+
 sub flush_namespace {
     my($self, $package) = @_;
     $package ||= $self->namespace;
 
-    no strict;
+    no strict 'refs';
     my $tab = \%{$package.'::'};
 
     for (keys %$tab) {
-	if(*{ $tab->{$_} }{CODE}) {
-	    undef_cv_if_owner($package, \&{ $tab->{$_} });
-	} 
-        if(*{ $tab->{$_} }{HASH}) {
-            undef %{ $tab->{$_} };
+        my $fullname = join '::', $package, $_;
+        #code/hash/array/scalar might be imported
+        #make sure the gv does not point elsewhere
+        #before undefing each
+        if (%$fullname) {
+            *{$fullname} = {};
+            undef %$fullname;
         }
-        if(*{ $tab->{$_} }{ARRAY}) {
-            undef @{ $tab->{$_} };
+        if (@$fullname) {
+            *{$fullname} = [];
+            undef @$fullname;
         }
-        if(*{ $tab->{$_} }{SCALAR}) {
-	    undef ${ $tab->{$_} };
+        if ($$fullname) {
+            my $tmp; #argh, no such thing as an anonymous scalar
+            *{$fullname} = \$tmp;
+            undef $$fullname;
         }
-     }
-}
-
-sub undef_cv_if_owner {
-    return unless $INC{'B.pm'};
-    my($package, $cv) = @_;
-    my $obj    = B::svref_2object($cv);
-    my $stash  = $obj->GV->STASH->NAME;
-    return unless $package eq $stash;
-    undef &$cv;
+        if (defined &$fullname) {
+            no warnings;
+            local $^W = 0;
+            if (my $p = prototype $fullname) {
+                *{$fullname} = eval "sub ($p) {}";
+            }
+            else {
+                *{$fullname} = sub {};
+            }
+	    undef &$fullname;
+	}
+        if (*{$fullname}{IO}) {
+            if (fileno $fullname) {
+                close $fullname;
+            }
+        }
+    }
 }
 
 1;
@@ -404,3 +418,4 @@ perl(1), mod_perl(3), Apache::Registry(3)
 
 Doug MacEachern
 
+=cut
