@@ -1,5 +1,5 @@
 use Apache ();
-use Apache::Constants qw(:server :common);
+use Apache::Constants qw(:server :common :methods);
 use Apache::test;
 use strict;
 
@@ -16,11 +16,11 @@ else {
 
 my $is_xs = ($r->uri =~ /_xs/);
 
-my $tests = 71;
+my $tests = 74;
 my $is_win32 = WIN32;
-$tests += 2 unless $is_win32;
+$tests += 4 unless $is_win32;
 my $test_get_set = Apache->can('set_handlers') && ($tests += 4);
-my $test_custom_response = (MODULE_MAGIC_NUMBER >= 19980324) && ($tests += 2);
+my $test_custom_response = (MODULE_MAGIC_NUMBER >= 19980324) && ($tests += 4);
 my $test_dir_config = $INC{'Apache/TestDirectives.pm'} && ($tests += 9);
 
 my $i;
@@ -68,6 +68,8 @@ unless ($is_win32) {
   print "Apache.pm == $ft_s, $0 == $ft_def\n";
   test ++$i, $ft_s != $ft_def;
   test ++$i, (-s $r->finfo) == $ft_def;
+  test ++$i, -T $r->finfo;
+  test ++$i, not -B $r->finfo;
 }
 
 my $the_request = $r->the_request;
@@ -157,6 +159,12 @@ test ++$i, $r->filename;
 test ++$i, $r->satisfies || 1;
 test ++$i, $r->some_auth_required || 1;
 
+$r->allowed(1 << M_GET);
+test ++$i, $r->allowed & (1 << M_GET);
+test ++$i, ! ($r->allowed & (1 << M_PUT));
+$r->allowed($r->allowed | (1 << M_PUT));
+test ++$i, $r->allowed & (1 << M_PUT);
+
 #dir_config
 
 my $c = $r->connection;
@@ -211,6 +219,8 @@ test ++$i, Apache->module("mod_perl.c");
 if($test_custom_response) {
     test ++$i, $r->custom_response(403, "no chance") || 1;
     test ++$i, $r->custom_response(403) =~ /chance/;
+    test ++$i, $r->custom_response(403, undef) || 1;
+    test ++$i, not defined $r->custom_response(403);
 }
 
 if($test_get_set) {
